import os

from forum.handler.BaseHandler import BaseHandler#跨域的类
from forum import manager#连接数据库的类
from forum.models import TopicModel,UserModel
from forum.wtforms import AddTopicForm
from forum.decorators import login_required_async#导入登录装饰器
from uuid import uuid4
from config import settings

#视频63显示主页帖子
class GetManyTopicHandler(BaseHandler):
    async def get(self):#get请求
        #从数据库获取数据  是一个Io操作  通过TopicModel查询
        topics = await manager.execute(TopicModel.select().order_by(TopicModel.create_time.desc()))
        rs_data = {}
        #封装一个data
        data = []
        for topic in topics:
            #把文章信息转成json 为什么要转为json数据
            t = topic.to_json()
            #把用户信息转为json
            t['user'] = t.get('user').to_json()
            #分割数据库中的图片地址 通过'，'分割
            #如果没有值
            if t.get('imgs') == None or t.get('imgs') == '':
                t['imgs'] = None
                #如果有值
            else:
                t['imgs'] =t.get('imgs').split(',')
            #78 增加返回评论数  comments是backup定义的
            t['comment_num'] = topic.comments.count()
            #增加返回收藏数  like_num是前端定义的词 在前端的home页面查找
            t['like_num'] = topic.collections.count()
            #添加数据
            data.append(t)
        rs_data['code'] = 200
        rs_data['msg'] = '获取成功'
        #返回data数据
        rs_data['topics'] = data
        self.finish(rs_data)
    
    #视频65  跟上面的一样 只不过修改sql语句
    async def post(self):
        #从前端获取传递过来的过滤类型
        type_ = self.get_body_argument('type')
         #从数据库获取数据  是一个Io操作
        topics = await manager.execute(TopicModel.select().where(TopicModel.type_ == type_).order_by(TopicModel.create_time.desc()))
        rs_data = {}
        data = []
        for topic in topics:
        #将topic转为json数据
            t = tran_topic(topic)
                #添加数据
            data.append(t)
        rs_data['code'] = 200
        rs_data['msg'] = '获取成功'
        rs_data['topics'] = data
        self.finish(rs_data)

#视频64 
class GetOneTopicHandler(BaseHandler):
    async def post(self):
        rs_data = {}
        #获取文章的数据  根据前端传的id
        id = self.get_body_argument('id')
        #从数据库中获取数据  
        topic = await manager.get(TopicModel,id = id)  #这是什么意思
        #将topic转为json数据
        t = tran_topic(topic)
        #传递给前端
        rs_data['code'] = 200
        rs_data['msg'] = '获取成功'
        rs_data['topic'] = t  #这里是topic没有s
        self.finish(rs_data)

#视频66             表示传递TopicModel数据
def tran_topic(topic:TopicModel) -> dict:
    #把文章信息转成json
    t = topic.to_json()
    #把用户信息转为json
    t['user'] = t.get('user').to_json()
    #分割图片地址 通过，分割
    if t.get('imgs') == None or t.get('imgs') == '':
        t['imgs'] = None
    else:
        t['imgs'] =t.get('imgs').split(',')  
    return t

#视频67
class AddTopicHandler(BaseHandler):
    @login_required_async
    async def post(self):
        #封装一个结果
        rs_data = {}
        #获取传递的数据，传递到Form  self.request.arguments表示前端传来的参数
        topic_form = AddTopicForm(self.request.arguments)
        #验证
        if topic_form.validate():
        #验证成功，增加数据
        #获取当前用户信息
            user = await manager.get(UserModel,id = self._user_id)
            #生成一个唯一的id
            id = uuid4()

            #建一个full_img_path
            img_path = []
            #保存图片 视频68  如果没有发图片 就给一个空列表
            imgs = self.request.files.get('imgs',[])
            for i in imgs:
                #获取文件名                   h后缀
                file_name = f'{uuid4().hex}{os.path.splitext(i.get("filename"))[-1]}'
                #获取文件全名  os.path.join表示路径的拼接到img文件夹中 新建一个叫file_name的文件
                full_path = os.path.join(settings.get('static_path'),'img',file_name)
                #保存文件 
                with open(full_path,'wb') as f:
                    f.write(i.get('body'))
                #追加图片路径到img_path中去
                img_path.append(f'/static/img/{file_name}')
            #更新到topic对象里面去                                                       ,分割开多个对象
            await manager.create(TopicModel,**topic_form.data,user = user,id=id,imgs=','.join(img_path))
            
            rs_data['code'] = 200
            rs_data['msg'] = '发帖成功'
        #验证失败，数据不合法
        else:
            rs_data['code'] = 500
            rs_data['msg'] = '发帖失败'
        #返回一个信息
        self.finish(rs_data)

# 视频69
class GetMyTopicHandler(BaseHandler):
    #登录装饰器 有次装饰器有tokenixnxi
    @login_required_async
    async def post(self):
        rs_data = {}
        #通过uid进行帅选获取帖子信息   通过TopicModel和UserModel这两张表关联查询  UserModel.id通过这个表的这个字段过滤条件
        topics = await manager.execute(TopicModel.select().join(UserModel).where(UserModel.id == self._user_id).order_by(TopicModel.create_time.desc()))
        #提示先登录
        # topics = await manager.execute(TopicModel.select().where(TopicModel.user.id == self._user_id).order_by(TopicModel.create_time.desc()))  bu不能用
        #将所有帖子信息换成json  包含user信息
        data = []
        for t in topics:
            #用这个方法可以把用户信息和图片信息转为json信息
            d = tran_topic(t)
            data.append(d)
        #响应给前端
        rs_data['code'] = 200
        rs_data['msg'] = '获取个人帖子信息成功'
        #显示所有帖子内容
        rs_data['topics'] = data
        self.finish(rs_data)

'''
自己增加热帖功能：
原理就是：将帖子按照点击数的多少显示
成功，就是显示哪一张图片，这里不应该显示图片
'''
class HotPostHandler(BaseHandler):
    async def get(self):#get请求
         #从数据库获取数据  是一个Io操作
        topics = await manager.execute(TopicModel.select().order_by(TopicModel.chick_num.desc()))
        rs_data = {}
        data = []
        for topic in topics:
            #把文章信息转成json
            t = topic.to_json()
            #把用户信息转为json
            t['user'] = t.get('user').to_json()
            '''
            #分割数据库中的图片地址 通过，分割
            if t.get('imgs') == None or t.get('imgs') == '':
                t['imgs'] = None
            else:
                t['imgs'] =t.get('imgs').split(',')
            #78 增加返回评论数  comments是backup定义的
            t['comment_num'] = topic.comments.count()
            # #增加返回收藏数  like_num是前端定义的词 在前端的home页面查找
            t['like_num'] = topic.collections.count()
            '''
                #添加数据
            data.append(t)
        rs_data['code'] = 200
        rs_data['msg'] = '获取热帖成功'
        rs_data['topics'] = data
        self.finish(rs_data)

#自己增加删除我的帖子的功能 成功
class DeleteMyTopicHandler(BaseHandler):
    #先登录
    @login_required_async
    async def post(self):
        topicId = self.get_body_argument('topicID')
        # print(topicId) #  ok
        #通过前端传过来的topciID定位帖子 再执行sql语句
        rs = await manager.execute(TopicModel.delete().where(TopicModel.id == topicId))
        if rs > 0:
            self.finish({'code':200,'msg':'删除帖子成功'})   
        else:
            self.finish({'code':500,'msg':'删除帖子失败'})   
        '''
         #通过userid进行删除
        rs = await manager.execute(TopicModel.delete().join(UserModel).where(UserModel.id == user_id))
        # print(rs)       
        if rs > 0:
            self.finish({'code':200,'msg':'删除帖子成功'})   
        else:
            self.finish({'code':500,'msg':'删除帖子失败'})    
        '''


