
from datetime import datetime
from tabnanny import verbose
from forum import database
from peewee import *

#定义一个全局性的数据库对象
#这样是基类 让下面创建的model继承它
class BaseModel(Model):
    #                                   这里的datetime.now不要加()
    create_time = DateTimeField(default=datetime.now,verbose_name = '创建时间')
    #定义一个公用的json方法  视频55
    def to_json(self):
        r = {}
        for k in self.__data__.keys():
            #判断属性是不是create_time datetime的数据会特别点不是字符串会报错
            if k == 'create_time':
                #若果是先转化类型
                r[k] = str(getattr(self,k))
            else:
            #getattr  python的一个方法，获取某个属性的值
                r[k] = getattr(self,k)#self表示当前对象signatrue
        return r
    class Meta:
        database = database

#这里继承model  创建的表
class UserModel(BaseModel):
    id= CharField(primary_key =True)
    email= CharField(max_length=32,verbose_name ='账号')
    nick_name= CharField(max_length=32,verbose_name ='昵称',null=True)
    #58  null=True表示可以为空
    gender= IntegerField(verbose_name='性别',null=True)
    password= CharField(max_length=128,verbose_name ='密码')
    signatrue= CharField(max_length=32,verbose_name ='签名',null=True)
    pic= CharField(max_length=100,verbose_name ='头像',null=True)
    #状态字段
    status= IntegerField(verbose_name='账号状态',default=1)

    class Meta:
        table_name = 't_user'

    # 如果发现表结构,需要添加字段.先备份数据sql,然后再重新建立表!!
    # 如果字段多了,能不删除就不删除

#视频62 创建文章表
class TopicModel(BaseModel):
    id= CharField(primary_key = True)
    title= CharField(verbose_name = '标题')
    imgs= CharField(max_length=1000,verbose_name='图片')
    content= CharField(verbose_name = '内容')
    chick_num= IntegerField(verbose_name = '点击数')
    type_= CharField(verbose_name = '类型')
    user= ForeignKeyField(UserModel,backref='topics')

    class Meta:
        table_name = 't_topic'

#70 评论表
class CommentModel(BaseModel):
    id = CharField(primary_key = True)
    content = CharField(verbose_name='内容')
    topic = ForeignKeyField(TopicModel,backref='comments')#生成的字段叫topic_id
#   topic = ForeignKeyField(TopicModel,backref='comments')
    user = ForeignKeyField(UserModel,backref='comments')#生成的字段叫user_id
#   user = ForeignKeyField(UserModel,backref='comments')
    class Meta:
        #设置表名
        table_name = 't_comment'

#74  收藏帖子功能
#1位用户 收藏多个帖子
#1个帖子可以被多个用户收藏
class CollectionModel(BaseModel):
    topic = ForeignKeyField(TopicModel,backref='collections')
    user = ForeignKeyField(UserModel,backref='collections')

    class Meta:
        table_name = 't_collection'
        #创建一个联合主键
        primary_key = CompositeKey('topic','user')

#79 用户关注 
'''
一位用户 可关注多位用户
一位用户 可被多个用户关注
张三关注李四 王五
李四关注张三 王五
王五被张三李四关注

自关联的多对多挂你
'''

class FollowModel(BaseModel):
    #backref表示反向查找
    to_user = ForeignKeyField(UserModel,backref='to_user')#关注谁
    from_user = ForeignKeyField(UserModel,backref='from_user')#谁关注
    class Meta:
        table_name='t_follow'
        primary_key = CompositeKey('to_user','from_user')#设置联合主键
